@@ -7,14 +7,16 @@ import hildon
 from os import listdir, path, system
 from ConfigParser import SafeConfigParser
 
-version = "1.4-1"
-autostart = False
+version = "1.5-0"
+stopTracking = True
+stopWhenIdle = False
+stopAfterMinutes = "5___"
+keepWifiPMOff = True
 confirmDisconnect = True
 showConnected = True
-stopTracking = False
-keepWifiPMOff = False
-skipSystemCheck = False
 connectedIcon = "Symbol"
+skipSystemCheck = False
+autostart = False
 
 class Main(object):
     def __init__(self):
@@ -22,11 +24,14 @@ class Main(object):
         global confirmDisconnect
         global showConnected
         global stopTracking
+        global stopWhenIdle
         global keepWifiPMOff
         global skipSystemCheck
+        global stopAfterMinutes
         global connectedIcon
+        
         autostartDisabled = False
-       
+        
         self.parser = SafeConfigParser()
         self.parser.optionxform = str
         if path.isfile("/home/user/.openssh-status-widget"):
@@ -40,17 +45,35 @@ class Main(object):
         if not path.isfile("/home/user/.openssh-status-widget"):
             configFile = open('/home/user/.openssh-status-widget', 'w')
             self.parser.add_section('options')
+            self.parser.set('options', 'stopTracking', str(stopTracking))
+            self.parser.set('options', 'stopWhenIdle', str(stopWhenIdle))
+            self.parser.set('options', 'stopAfterMinutes', str(connectedIcon))
+            self.parser.set('options', 'keepWifiPMOff', str(keepWifiPMOff))
             self.parser.set('options', 'confirmDisconnect', str(confirmDisconnect))
             self.parser.set('options', 'showConnected', str(showConnected))
-            self.parser.set('options', 'stopTracking', str(stopTracking))
-            self.parser.set('options', 'keepWifiPMOff', str(keepWifiPMOff))
-            self.parser.set('options', 'skipSystemCheck', str(skipSystemCheck))
             self.parser.set('options', 'connectedIcon', str(connectedIcon))
+            self.parser.set('options', 'skipSystemCheck', str(skipSystemCheck))
             self.parser.write(configFile)
             configFile.close()
         else:
             self.parser.read('/home/user/.openssh-status-widget')
             if self.parser.has_section('options'):
+                if self.parser.has_option('options', 'stopTracking'):
+                    stopTracking = self.parser.getboolean('options', 'stopTracking')
+                else:
+                    self.parser.set('options', 'stopTracking', str(stopTracking))
+                if self.parser.has_option('options', 'stopWhenIdle'):
+                    stopWhenIdle = self.parser.getboolean('options', 'stopWhenIdle')
+                else:
+                    self.parser.set('options', 'stopWhenIdle', str(stopWhenIdle))
+                if self.parser.has_option('options', 'stopAfterMinutes'):
+                    stopAfterMinutes = self.parser.get('options', 'stopAfterMinutes')
+                else:
+                    self.parser.set('options', 'stopAfterMinutes', str(stopAfterMinutes))
+                if self.parser.has_option('options', 'keepWifiPMOff'):
+                    keepWifiPMOff = self.parser.getboolean('options', 'keepWifiPMOff')
+                else:
+                    self.parser.set('options', 'keepWifiPMOff', str(keepWifiPMOff))
                 if self.parser.has_option('options', 'confirmDisconnect'):
                     confirmDisconnect = self.parser.getboolean('options', 'confirmDisconnect')
                 else:
@@ -59,22 +82,14 @@ class Main(object):
                     showConnected = self.parser.getboolean('options', 'showConnected')
                 else:
                     self.parser.set('options', 'showConnected', str(showConnected))
-                if self.parser.has_option('options', 'stopTracking'):
-                    stopTracking = self.parser.getboolean('options', 'stopTracking')
-                else:
-                    self.parser.set('options', 'stopTracking', str(stopTracking))
-                if self.parser.has_option('options', 'keepWifiPMOff'):
-                    keepWifiPMOff = self.parser.getboolean('options', 'keepWifiPMOff')
-                else:
-                    self.parser.set('options', 'keepWifiPMOff', str(keepWifiPMOff))
-                if self.parser.has_option('options', 'skipSystemCheck'):
-                    skipSystemCheck = self.parser.getboolean('options', 'skipSystemCheck')
-                else:
-                    self.parser.set('options', 'skipSystemCheck', str(skipSystemCheck))
                 if self.parser.has_option('options', 'connectedIcon'):
                     connectedIcon = self.parser.get('options', 'connectedIcon')
                 else:
                     self.parser.set('options', 'connectedIcon', str(connectedIcon))
+                if self.parser.has_option('options', 'skipSystemCheck'):
+                    skipSystemCheck = self.parser.getboolean('options', 'skipSystemCheck')
+                else:
+                    self.parser.set('options', 'skipSystemCheck', str(skipSystemCheck))
         
         if not path.isfile("/etc/event.d/sshd"):
             system("sudo /usr/sbin/openssh-status.sh create_sshd")
@@ -89,13 +104,15 @@ class Main(object):
         if not autostart and not autostartDisabled:
             system("sudo /usr/sbin/openssh-status.sh insert_start")
             
-        self.list = []
+        self.indicatorList = []
         for f in listdir('/usr/share/icons/hicolor/18x18/hildon/'):
             if f[0:21] == "openssh-status_users_" and f <> "openssh-status_users_symbol.png" and f[-4:] ==".png":
                 f = f[21:-4]
-                self.list.append(f.title())
-        self.list.sort()
-        self.list.insert(0, "Symbol")
+                self.indicatorList.append(f.title())
+        self.indicatorList.sort()
+        self.indicatorList.insert(0, "Symbol")
+        
+        self.timeoutList = ["1", "5", "15", "30", "60"]
         
         self.main()
     
@@ -105,13 +122,25 @@ class Main(object):
         window.connect("delete_event", gtk.main_quit, None)
         
         labelWidget = gtk.Label("Widget options:")
-        labelOther = gtk.Label("Other options:")
+        labelOther = gtk.Label("Server options:")
+    
+        self.buttonStopTracking = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonStopTracking.set_label("Stop tracking status when off")
+    
+        self.buttonStopWhenIdle = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonStopWhenIdle.set_label("Stop server when not connected")
+    
+        self.buttonStopAfterMinutes = hildon.PickerButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL | hildon.BUTTON_STYLE_NORMAL)
+        self.buttonStopAfterMinutes.set_title("Minutes to wait:")
+        self.buttonStopAfterMinutes.set_alignment(0.09, 0.5, 0, 0)
+        self.buttonStopAfterMinutes.set_selector(self.selectTimeout())
+        try:
+            self.buttonStopAfterMinutes.set_active(self.timeoutList.index(stopAfterMinutes.strip('_')))
+        except:
+            self.buttonConnectedIcon.set_active(0)
     
-        self.buttonAutostart = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
-        self.buttonAutostart.set_label("Run server at system startup")
-                
         self.buttonKeepWifiPMOff = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
-        self.buttonKeepWifiPMOff.set_label("Keep WiFi power management off when connected")
+        self.buttonKeepWifiPMOff.set_label("Keep wifi power management off when connected")
     
         self.buttonConfirm = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.buttonConfirm.set_label("Confirm when disconnecting")
@@ -124,25 +153,27 @@ class Main(object):
         self.buttonConnectedIcon.set_alignment(0.09, 0.5, 0, 0)
         self.buttonConnectedIcon.set_selector(self.selectIndicator())
         try:
-            self.buttonConnectedIcon.set_active(self.list.index(connectedIcon.strip('_')))
+            self.buttonConnectedIcon.set_active(self.indicatorList.index(connectedIcon.strip('_')))
         except:
             self.buttonConnectedIcon.set_active(0)
     
-        self.buttonStopTracking = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
-        self.buttonStopTracking.set_label("Stop tracking status when off")
-    
+        self.buttonAutostart = hildon.CheckButton(gtk.HILDON_SIZE_FULLSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.buttonAutostart.set_label("Run server at system startup")
+                
         vboxBorder = gtk.VBox(False, 0)
         vboxBorder.set_border_width(8)
         vbox = gtk.VBox(False, 0)
         vbox.set_border_width(0)
-        vbox.pack_start(labelWidget, False, False, 12)
+        vbox.pack_start(labelWidget, False, False, 8)
+        vbox.pack_start(self.buttonStopTracking, False, False, 0)
+        vbox.pack_start(self.buttonStopWhenIdle, False, False, 0)
+        vbox.pack_start(self.buttonStopAfterMinutes, False, False, 0)
+        vbox.pack_start(self.buttonKeepWifiPMOff, False, False, 0)
         vbox.pack_start(self.buttonConfirm, False, False, 0)
         vbox.pack_start(self.buttonShowConnected, False, False, 0)
         vbox.pack_start(self.buttonConnectedIcon, False, False, 0)
-        vbox.pack_start(self.buttonStopTracking, False, False, 0)
-        vbox.pack_start(labelOther, False, False, 12)
+        vbox.pack_start(labelOther, False, False, 8)
         vbox.pack_start(self.buttonAutostart, False, False, 0)
-        vbox.pack_start(self.buttonKeepWifiPMOff, False, False, 0)
         
         areaScrolling = hildon.PannableArea()
         areaScrolling.add_with_viewport(vbox)
@@ -152,10 +183,20 @@ class Main(object):
         
         window.add(vboxBorder)
         
-        if autostart:
-            self.buttonAutostart.set_active(True)
+        if stopTracking:
+            self.buttonStopTracking.set_active(True)
         else:
-            self.buttonAutostart.set_active(False)
+            self.buttonStopTracking.set_active(False)
+        if stopWhenIdle:
+            self.buttonStopWhenIdle.set_active(True)
+            self.buttonStopAfterMinutes.set_sensitive(True)
+        else:
+            self.buttonStopWhenIdle.set_active(False)
+            self.buttonStopAfterMinutes.set_sensitive(False)
+        if keepWifiPMOff:
+            self.buttonKeepWifiPMOff.set_active(True)
+        else:
+            self.buttonKeepWifiPMOff.set_active(False)
         if confirmDisconnect:
             self.buttonConfirm.set_active(True)
         else:
@@ -166,22 +207,20 @@ class Main(object):
         else:
             self.buttonShowConnected.set_active(False)
             self.buttonConnectedIcon.set_sensitive(False)
-        if stopTracking:
-            self.buttonStopTracking.set_active(True)
+        if autostart:
+            self.buttonAutostart.set_active(True)
         else:
-            self.buttonStopTracking.set_active(False)
-        if keepWifiPMOff:
-            self.buttonKeepWifiPMOff.set_active(True)
-        else:
-            self.buttonKeepWifiPMOff.set_active(False)
+            self.buttonAutostart.set_active(False)
             
             
-        self.buttonAutostart.connect("clicked", self.buttonAutostart_clicked)
+        self.buttonStopTracking.connect("clicked", self.buttonStopTracking_clicked)
+        self.buttonStopWhenIdle.connect("clicked", self.buttonStopWhenIdle_clicked)
+        self.buttonStopAfterMinutes.connect("value-changed", self.buttonStopAfterMinutes_changed)
+        self.buttonKeepWifiPMOff.connect("clicked", self.buttonKeepWifiPMOff_clicked)
         self.buttonConfirm.connect("clicked", self.buttonConfirm_clicked)
         self.buttonShowConnected.connect("clicked", self.buttonShowConnected_clicked)
         self.buttonConnectedIcon.connect("value-changed", self.buttonConnectedIcon_changed)
-        self.buttonStopTracking.connect("clicked", self.buttonStopTracking_clicked)
-        self.buttonKeepWifiPMOff.connect("clicked", self.buttonKeepWifiPMOff_clicked)
+        self.buttonAutostart.connect("clicked", self.buttonAutostart_clicked)
         
         menu = self.create_menu()
         window.set_app_menu(menu)
@@ -190,14 +229,41 @@ class Main(object):
     
         gtk.main()
         
-    def buttonAutostart_clicked(self, dummy):
-        global autostart
-        autostart = not autostart
-        if autostart:
-            system("sudo /usr/sbin/openssh-status.sh enable_autostart")
+    def buttonStopTracking_clicked(self, dummy):
+        global stopTracking
+        stopTracking = not stopTracking
+        self.parser.set('options', 'stopTracking', str(stopTracking))
+        self.write_config()
+        
+    def buttonStopWhenIdle_clicked(self, dummy):
+        global stopWhenIdle
+        stopWhenIdle = not stopWhenIdle
+        if stopWhenIdle:
+            self.buttonStopAfterMinutes.set_sensitive(True)
         else:
-            system("sudo /usr/sbin/openssh-status.sh disable_autostart")
-
+            self.buttonStopAfterMinutes.set_sensitive(False)
+        self.parser.set('options', 'stopWhenIdle', str(stopWhenIdle))
+        self.write_config()
+        
+    def selectTimeout(self):
+        selector = hildon.TouchSelector(text = True)
+        for i in self.timeoutList:
+            selector.append_text(i)
+        return selector
+        
+    def buttonStopAfterMinutes_changed(self, dummy):
+        minutes = self.buttonStopAfterMinutes.get_value()
+        while len(minutes) < 4:
+            minutes = minutes + "_"
+        self.parser.set('options', 'stopAfterMinutes', minutes)
+        self.write_config()
+        
+    def buttonKeepWifiPMOff_clicked(self, dummy):
+        global keepWifiPMOff
+        keepWifiPMOff = not keepWifiPMOff
+        self.parser.set('options', 'keepWifiPMOff', str(keepWifiPMOff))
+        self.write_config()
+        
     def buttonConfirm_clicked(self, dummy):
         global confirmDisconnect
         confirmDisconnect = not confirmDisconnect
@@ -216,7 +282,7 @@ class Main(object):
         
     def selectIndicator(self):
         selector = hildon.TouchSelector(text = True)
-        for i in self.list:
+        for i in self.indicatorList:
             selector.append_text(i)
         return selector
         
@@ -227,18 +293,14 @@ class Main(object):
         self.parser.set('options', 'connectedIcon', style)
         self.write_config()
         
-    def buttonStopTracking_clicked(self, dummy):
-        global stopTracking
-        stopTracking = not stopTracking
-        self.parser.set('options', 'stopTracking', str(stopTracking))
-        self.write_config()
-        
-    def buttonKeepWifiPMOff_clicked(self, dummy):
-        global keepWifiPMOff
-        keepWifiPMOff = not keepWifiPMOff
-        self.parser.set('options', 'keepWifiPMOff', str(keepWifiPMOff))
-        self.write_config()
-        
+    def buttonAutostart_clicked(self, dummy):
+        global autostart
+        autostart = not autostart
+        if autostart:
+            system("sudo /usr/sbin/openssh-status.sh enable_autostart")
+        else:
+            system("sudo /usr/sbin/openssh-status.sh disable_autostart")
+
     def write_config(self):
         configFile = open('/home/user/.openssh-status-widget', 'r+')
         self.parser.write(configFile)
