@@ -7,36 +7,44 @@ import hildon
 import subprocess
 import time
 from os import path
+from commands import getoutput
 from ConfigParser import SafeConfigParser
 
 class opensshStatus(hildondesktop.StatusMenuItem):
     def __init__(self):
         hildondesktop.StatusMenuItem.__init__(self)
 
+        self.stopTracking = True
+        self.stopWhenIdle = False
+        self.stopAfterMinutes = 5
+        self.keepWifiPMOff = True
         self.confirmDisconnect = True
         self.showConnected = True
-        self.stopTracking = False
-        self.keepWifiPMOff = True
-        self.skipSystemCheck = False
         connectedIcon = "symbol"
+        self.skipSystemCheck = False
+        
         if path.isfile("/home/user/.openssh-status-widget"):
             parser = SafeConfigParser()
             try:
                 parser.read('/home/user/.openssh-status-widget')
             except:
                 subprocess.call(["rm", "/home/user/.openssh-status-widget"])
+            if parser.has_option('options', 'stopTracking'):
+                self.stopTracking = parser.getboolean('options', 'stopTracking')
+            if parser.has_option('options', 'stopWhenIdle'):
+                self.stopWhenIdle = parser.getboolean('options', 'stopWhenIdle')
+            if parser.has_option('options', 'stopAfterMinutes'):
+                self.stopAfterMinutes = int(parser.get('options', 'stopAfterMinutes').strip('_'))
+            if parser.has_option('options', 'keepWifiPMOff'):
+                self.keepWifiPMOff = parser.getboolean('options', 'keepWifiPMOff')
             if parser.has_option('options', 'confirmDisconnect'):
                 self.confirmDisconnect = parser.getboolean('options', 'confirmDisconnect')
             if parser.has_option('options', 'showConnected'):
                 self.showConnected = parser.getboolean('options', 'showConnected')
-            if parser.has_option('options', 'stopTracking'):
-                self.stopTracking = parser.getboolean('options', 'stopTracking')
-            if parser.has_option('options', 'keepWifiPMOff'):
-                self.keepWifiPMOff = parser.getboolean('options', 'keepWifiPMOff')
-            if parser.has_option('options', 'skipSystemCheck'):
-                self.skipSystemCheck = parser.getboolean('options', 'skipSystemCheck')
             if parser.has_option('options', 'connectedIcon'):
                 connectedIcon = parser.get('options', 'connectedIcon')
+            if parser.has_option('options', 'skipSystemCheck'):
+                self.skipSystemCheck = parser.getboolean('options', 'skipSystemCheck')
             
         self.icon=gtk.gdk.pixbuf_new_from_file("/usr/share/icons/hicolor/18x18/hildon/openssh-status_no-users.png")
         try:
@@ -57,8 +65,10 @@ class opensshStatus(hildondesktop.StatusMenuItem):
 
         self.on = False
         self.connected = False
+        self.waitWhileIdle = False
         self.refresh = False
         self.refreshWifiPMOff = False
+        self.labelConnected = 0
         self.sshJustOff = 0
         
         self.start()
@@ -91,7 +101,9 @@ class opensshStatus(hildondesktop.StatusMenuItem):
             else:
                 self.set_status_area_icon(self.icon)
             self.refresh = True
-            gobject.timeout_add(2000, self.sshTracker)
+            if not self.sshConnected:
+                self.button.set_text("OpenSSH Server", "0 connections")
+            gobject.timeout_add(5000, self.sshTracker)
         else:
             self.button.set_image(self.image_off)
             if self.sshConnected:
@@ -101,13 +113,13 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                 else:
                     self.set_status_area_icon(self.icon)
                 self.refresh = True
-                gobject.timeout_add(2000, self.sshTracker)
+                gobject.timeout_add(5000, self.sshTracker)
             else:
                 self.set_status_area_icon(None)
                 self.button.set_text("OpenSSH Server", "Off")
                 if not self.stopTracking:
                     self.refresh = True
-                    gobject.timeout_add(2000, self.sshTracker)
+                    gobject.timeout_add(5000, self.sshTracker)
             
     def adjustSshFile(self):
         self.dialog = gtk.Dialog("OpenSSH status", None)
@@ -153,22 +165,14 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         message.destroy()
         
     def sshStatus(self):
-        shellCommand1=subprocess.Popen(["ps", "aux"], stdout=subprocess.PIPE)
-        shellCommand2=subprocess.Popen(["grep", "-c", "/usr/sbin/sshd"], stdin=shellCommand1.stdout, stdout=subprocess.PIPE)
-        shellCommand1.stdout.close()
-        self.sshRunning=int(shellCommand2.communicate()[0])-1
+        self.sshRunning = int(getoutput("ps aux | grep -c /usr/sbin/sshd")) - 2
         
     def sshUsers(self):
-        shellCommand1=subprocess.Popen(["ps", "aux"], stdout=subprocess.PIPE)
-        shellCommand2=subprocess.Popen(["grep", "sshd:"], stdin=shellCommand1.stdout, stdout=subprocess.PIPE)
-        shellCommand3=subprocess.Popen(["grep", "-c", "@"], stdin=shellCommand2.stdout, stdout=subprocess.PIPE)
-        shellCommand1.stdout.close()
-        shellCommand2.stdout.close()
-        self.sshConnected = int(shellCommand3.communicate()[0])
+        self.sshConnected = int(getoutput("ps aux | grep sshd: | grep -c @")) - 1
         
     def sshTracker(self):
-        self.sshStatus()
-        self.sshUsers()
+        self.sshRunning = int(getoutput("ps aux | grep -c /usr/sbin/sshd")) - 2
+        self.sshConnected = int(getoutput("ps aux | grep sshd: | grep -c @")) - 1
         if self.sshRunning and self.sshJustOff:
             self.sshJustOff = 0
         if not self.sshRunning and self.on:
@@ -197,37 +201,59 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         elif self.refreshWifiPMOff:
             self.refreshWifiPMOff = False
             subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "enable_wifipm"])
-        if self.sshConnected == 1:
-            self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connection")
-            return True
-        elif self.sshConnected > 1 or (not self.sshConnected and self.sshRunning):
-            self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connections")
-            return True
-        else:
-            if self.sshJustOff < 1:
-                self.sshJustOff += 1
+        if self.sshConnected != self.labelConnected:
+            self.labelConnected = self.sshConnected
+            self.waitWhileIdle = False
+            if self.sshConnected == 1:
+                self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connection")
                 return True
-            else:
-                self.on = False
-                self.button.set_image(self.image_off)
-                self.set_status_area_icon(None)
-                self.button.set_text("OpenSSH Server", "Off")
-                if self.stopTracking:
-                    if self.sshJustOff < 5:
-                        self.sshJustOff += 1
-                        return True
-                    else:
-                        self.sshJustOff = 0
-                        self.refresh = False
-                        return False
-                else:
+            elif self.sshConnected > 1 or (not self.sshConnected and self.sshRunning):
+                self.button.set_text("OpenSSH Server", str(self.sshConnected)+" connections")
+                return True
+        if not self.sshConnected and not self.sshRunning:
+            #if self.sshJustOff < 1:
+            #    self.sshJustOff += 1
+            #    return True
+            #else:
+            self.on = False
+            self.button.set_image(self.image_off)
+            self.set_status_area_icon(None)
+            self.button.set_text("OpenSSH Server", "Off")
+            if self.stopTracking:
+                if self.sshJustOff < 3:
+                    self.sshJustOff += 1
                     return True
+                else:
+                    self.sshJustOff = 0
+                    self.refresh = False
+                    self.waitWhileIdle = False
+                    return False
+            else:
+                return True
+        if not self.sshConnected and self.stopWhenIdle:
+            if not self.waitWhileIdle:
+                self.idleStartTime = time.time()
+                self.waitWhileIdle = True
+                return True
+            elif time.time() - self.idleStartTime + 5 > self.stopAfterMinutes * 60:
+                subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "stop_sshd"])
+                time.sleep(1)
+                self.sshStatus()
+                if self.sshRunning:
+                    self.printDialog("Could NOT stop OpenSSH Server!")
+                else:
+                    self.on = False
+                    self.waitWhileIdle = False
+                    self.button.set_text("OpenSSH Server", "Off")
+                    self.button.set_image(self.image_off)
+                    self.set_status_area_icon(None)
+        return True
         
     def buttonClicked(self, dummy):
         self.sshStatus()
         if self.sshRunning and self.on:
             subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "stop_sshd"])
-            time.sleep(0.5)
+            time.sleep(1)
             self.sshStatus()
             if self.sshRunning:
                 self.printDialog("Could NOT stop OpenSSH Server!")
@@ -256,11 +282,11 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                 self.set_status_area_icon(self.icon)
             if not self.refresh:
                 self.refresh = True
-                gobject.timeout_add(2000, self.sshTracker)
+                gobject.timeout_add(5000, self.sshTracker)
             self.printDialog("OpenSSH Server already running! Display updated.")
         else:
             subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "start_sshd"])
-            time.sleep(0.5)
+            time.sleep(1)
             self.sshStatus()
             self.sshUsers()
             if self.sshRunning:
@@ -272,7 +298,7 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                     self.set_status_area_icon(self.icon)
                 if not self.refresh:
                     self.refresh = True
-                    gobject.timeout_add(2000, self.sshTracker)
+                    gobject.timeout_add(5000, self.sshTracker)
             else:
                 self.printDialog("Could NOT start OpenSSH Server!")
 
@@ -298,7 +324,7 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         elif self.sshRunning or self.sshConnected:
             self.sshDisconnect()
         else:
-            time.sleep(0.5)
+            time.sleep(1)
             self.printDialog("OpenSSH Server already stopped!")
         
     def sshDisconnect(self):
@@ -306,7 +332,7 @@ class opensshStatus(hildondesktop.StatusMenuItem):
         self.sshUsers()
         if self.sshRunning:
             subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "kill_sshd"])
-            time.sleep(0.5)
+            time.sleep(1)
             self.sshStatus()
             self.sshUsers()
             if self.sshRunning:
@@ -320,7 +346,7 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                 self.set_status_area_icon(None)
         elif self.sshConnected:
             subprocess.call(["sudo", "/usr/sbin/openssh-status.sh", "kill_sshd"])
-            time.sleep(0.5)
+            time.sleep(1)
             self.sshUsers()
             if self.sshConnected:
                 self.printDialog("Could NOT close OpenSSH connections!")
@@ -330,7 +356,7 @@ class opensshStatus(hildondesktop.StatusMenuItem):
                 self.button.set_image(self.image_off)
                 self.set_status_area_icon(None)
         else:
-            time.sleep(0.5)
+            time.sleep(1)
             self.printDialog("OpenSSH Server already stopped!")
             
     def printDialog(self, msg):
